@@ -192,6 +192,14 @@ impl Capacity {
             Capacity::Range(_whole, _decimal) => None,
         }
     }
+
+    pub fn sql_format(&self) -> String {
+        match *self {
+            Capacity::Limit(limit) => format!("({})", limit),
+            Capacity::Range(_whole, _decimal) => format!("({}, {})", _whole, _decimal),
+        }
+    }
+
 }
 
 #[derive(Debug, PartialEq, Clone)]
@@ -202,6 +210,16 @@ pub enum ColumnConstraint {
     AutoIncrement(Option<String>),
 }
 
+impl ColumnConstraint {
+    pub fn sql_format(&self) -> String {
+        match self {
+            ColumnConstraint::NotNull => "not null".into(),
+            ColumnConstraint::DefaultValue(v) => v.sql_format(), 
+            ColumnConstraint::AutoIncrement(_) => "auto_increment".into(),
+        }
+    }
+}
+
 
 #[derive(Debug, PartialEq, Clone)]
 pub enum Literal {
@@ -221,6 +239,22 @@ pub enum Literal {
     ArrayString(Vec<String>),
 }
 
+impl Literal {
+    pub fn sql_format(&self) -> String {
+        match self {
+            Literal::Bool(v) => v.to_string(),
+            Literal::Integer(v) => v.to_string(),
+            Literal::Double(v) => v.to_string(),
+            Literal::Uuid(v) => v.to_string(),
+            Literal::String(v) => v.to_owned(),
+            Literal::Blob(v) => String::from_utf8(v.to_owned()).unwrap_or_default(),
+            Literal::CurrentTime => "now()".to_string(),
+            Literal::CurrentDate => "now()".to_string(),
+            _ => String::default(),
+        }
+    }
+}
+
 #[derive(Debug, PartialEq, Clone)]
 pub struct ColumnStat {
     pub avg_width: i32, /* average width of the column, (the number of characters) */
