@@ -119,8 +119,11 @@ mod types;
 mod database;
 mod platform;
 mod data;
+#[cfg(feature = "akita-auth")]
 mod auth;
 mod manager;
+#[allow(unused)]
+mod fuse;
 
 
 #[doc(inline)]
@@ -139,7 +142,10 @@ pub use pool::{AkitaConfig, LogLevel, Pool};
 pub use data::*;
 #[doc(inline)]
 pub use information::*;
+#[cfg(feature = "akita-auth")]
 pub use auth::*;
+pub use fuse::*;
+pub use types::SqlType;
 #[doc(inline)]
 pub use manager::{AkitaEntityManager, AkitaManager};
 pub use crate as akita;
